@@ -42,8 +42,16 @@ gdal_chooseInstallation <- function(hasDrivers)
 				function(x,hasDrivers)
 				{
 					installation_drivers <- x$drivers
-					return(length(intersect(hasDrivers,installation_drivers$format_code)) 
-						== length(hasDrivers))
+					if(!grepl("^2",x$version))
+					{
+						return(length(intersect(hasDrivers,installation_drivers$format_code)) 
+										== length(hasDrivers))
+					} else
+					{
+						# FIX FOR VERSION 2.0						
+						matching_drivers <- grep(paste(hasDrivers,"-",sep=""),installation_drivers$format_code)
+						return(length(matching_drivers)==length(hasDrivers))
+					}				
 				},
 				hasDrivers=hasDrivers)	
 		current_match <- current_match&driver_match
